import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";

var _DISPLAY;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import React, { useCallback } from 'react';
import PropTypes from 'prop-types';
import { field } from '../prop-types';
import MoleculeInputField from '@s-ui/react-molecule-input-field';
import { FIELDS, DISPLAYS } from '../Standard'; // mapping between DSL standard and HTML input type attribute

var DISPLAY = (_DISPLAY = {}, _DISPLAY[DISPLAYS[FIELDS.TEXT].EMAIL] = 'email', _DISPLAY[DISPLAYS[FIELDS.TEXT].PHONE] = 'tel', _DISPLAY[DISPLAYS[FIELDS.TEXT].TEXT] = 'text', _DISPLAY[DISPLAYS[FIELDS.TEXT].DEFAULT] = 'text', _DISPLAY);

var Input = function Input(_ref) {
  var input = _ref.input,
      tabIndex = _ref.tabIndex,
      onChange = _ref.onChange,
      size = _ref.size,
      leftAddon = _ref.leftAddon,
      rightAddon = _ref.rightAddon,
      errors = _ref.errors;
  var errorMessages = errors[input.id];
  var onChangeCallback = useCallback(function (evt) {
    return onChange(input.id, evt.target.value);
  }, [input, onChange]);
  var nextProps = {};

  switch (input.type) {
    case FIELDS.TEXT:
      {
        nextProps = {
          type: DISPLAY[input.display] || 'text'
        }; // this is the html input type (not the field type related to DSL form-builder)

        var constraints = input.constraints || [];
        nextProps = constraints.reduce(function (acc, constraint) {
          var _constraint$property, _constraint$property2, _constraint$property3, _constraint$property4, _constraint$property5, _constraint$property6, _constraint$property7;

          return _objectSpread({}, acc, {}, (constraint === null || constraint === void 0 ? void 0 : (_constraint$property = constraint.property) === null || _constraint$property === void 0 ? void 0 : _constraint$property.maxlength) && {
            maxLength: constraint === null || constraint === void 0 ? void 0 : (_constraint$property2 = constraint.property) === null || _constraint$property2 === void 0 ? void 0 : _constraint$property2.maxlength
          }, {}, (constraint === null || constraint === void 0 ? void 0 : (_constraint$property3 = constraint.property) === null || _constraint$property3 === void 0 ? void 0 : _constraint$property3.minlength) && {
            minLength: constraint === null || constraint === void 0 ? void 0 : (_constraint$property4 = constraint.property) === null || _constraint$property4 === void 0 ? void 0 : _constraint$property4.minlength
          }, {}, (constraint === null || constraint === void 0 ? void 0 : (_constraint$property5 = constraint.property) === null || _constraint$property5 === void 0 ? void 0 : _constraint$property5.notnull) === '' && {
            required: true
          }, {}, (constraint === null || constraint === void 0 ? void 0 : (_constraint$property6 = constraint.property) === null || _constraint$property6 === void 0 ? void 0 : _constraint$property6.pattern) && {
            pattern: constraint === null || constraint === void 0 ? void 0 : (_constraint$property7 = constraint.property) === null || _constraint$property7 === void 0 ? void 0 : _constraint$property7.pattern
          });
        }, nextProps);
        break;
      }

    case FIELDS.NUMERIC:
      {
        nextProps = {
          type: 'number'
        }; // this is the html type (not the field type related to DSL form-builder)

        var _constraints = input.constraints || [];

        nextProps = _constraints.reduce(function (acc, constraint) {
          var _constraint$property8, _constraint$property9, _constraint$property10, _constraint$property11, _constraint$property12, _constraint$property13;

          return _objectSpread({}, acc, {}, (constraint === null || constraint === void 0 ? void 0 : (_constraint$property8 = constraint.property) === null || _constraint$property8 === void 0 ? void 0 : _constraint$property8.max) && {
            max: constraint === null || constraint === void 0 ? void 0 : (_constraint$property9 = constraint.property) === null || _constraint$property9 === void 0 ? void 0 : _constraint$property9.max
          }, {}, (constraint === null || constraint === void 0 ? void 0 : (_constraint$property10 = constraint.property) === null || _constraint$property10 === void 0 ? void 0 : _constraint$property10.min) && {
            min: constraint === null || constraint === void 0 ? void 0 : (_constraint$property11 = constraint.property) === null || _constraint$property11 === void 0 ? void 0 : _constraint$property11.min
          }, {}, (constraint === null || constraint === void 0 ? void 0 : (_constraint$property12 = constraint.property) === null || _constraint$property12 === void 0 ? void 0 : _constraint$property12.notnull) === '' && {
            required: constraint === null || constraint === void 0 ? void 0 : (_constraint$property13 = constraint.property) === null || _constraint$property13 === void 0 ? void 0 : _constraint$property13.notnull
          });
        }, nextProps);
        break;
      }
  }

  nextProps = _objectSpread({}, nextProps, {}, input.hidden && {
    hidden: true
  }, {}, input.disabled && {
    disabled: true
  }, {}, !!errorMessages && {
    errorText: errorMessages.join('\n')
  });

  var inputProps = _objectSpread({
    placeholder: input.hint,
    id: input.id,
    name: input.id,
    label: input.label || '',
    tabIndex: tabIndex,
    value: input.value || '',
    onChange: onChangeCallback,
    size: size,
    leftAddon: leftAddon,
    rightAddon: rightAddon
  }, nextProps);

  if (!nextProps.type) {
    return React.createElement("span", null, "Unknown Input type ", input.type);
  }

  if (nextProps.hidden) {
    return null;
  }

  return React.createElement("div", {
    className: "sui-FormBuilder-field sui-FormBuilder-Input sui-FormBuilder-" + inputProps.id
  }, React.createElement(MoleculeInputField, inputProps));
};

Input.displayName = 'Input';
export default Input;