import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import React, { useCallback, useState } from 'react';
import PropTypes from 'prop-types';
import { field } from '../../prop-types';
import MoleculeAutosuggestField from '@s-ui/react-molecule-autosuggest-field';
import MoleculeAutosuggestOption from '@s-ui/react-molecule-dropdown-option';
import IconClose from '../../Icons/IconClose';

var fromValueToText = function fromValueToText(datalist) {
  return function (value) {
    var item = datalist.find(function (item) {
      return item.value === value;
    });
    return item === null || item === void 0 ? void 0 : item.text;
  };
};

var fromTextToValue = function fromTextToValue(datalist) {
  return function (text) {
    var item = datalist.find(function (item) {
      return item.text.toLowerCase() === text.toLowerCase();
    });
    return item === null || item === void 0 ? void 0 : item.value;
  };
};

var AutosuggestSelect = function AutosuggestSelect(_ref) {
  var select = _ref.select,
      tabIndex = _ref.tabIndex,
      onChange = _ref.onChange,
      size = _ref.size,
      errors = _ref.errors;
  var errorMessages = errors[select.id];
  var _select$datalist = select.datalist,
      datalist = _select$datalist === void 0 ? [] : _select$datalist;
  var fromTextToValueWithDatalist = fromTextToValue(datalist);
  var fromValueToTextWithDatalist = fromValueToText(datalist);
  var text = fromValueToTextWithDatalist(select.value);

  var _useState = useState(text),
      stateText = _useState[0],
      setStateText = _useState[1];

  var onChangeCallback = useCallback(function (evt, _ref2) {
    var text = _ref2.value;
    var value = fromTextToValueWithDatalist(text);
    setStateText(text);
    onChange(select.id, value);
  }, [fromTextToValueWithDatalist, onChange, select]); // transform constraints to props

  var constraints = select.constraints || [];
  var constraintsProps = {};
  constraintsProps = constraints.reduce(function (acc, constraint) {
    var _constraint$property;

    if ((constraint === null || constraint === void 0 ? void 0 : (_constraint$property = constraint.property) === null || _constraint$property === void 0 ? void 0 : _constraint$property.notnull) === '') {
      return _objectSpread({}, acc, {
        required: true
      });
    } else {
      return acc;
    }
  }, constraintsProps);

  var autosuggestProps = _objectSpread({
    id: select.id,
    label: select.label,
    name: select.name,
    placeholder: select.hint,
    onChange: onChangeCallback,
    iconClear: React.createElement(IconClose, null),
    value: stateText,
    tabIndex: tabIndex
  }, select.disabled && {
    disabled: true
  }, {}, select.hidden && {
    hidden: true
  }, {}, !!errorMessages && {
    errorText: errorMessages.join('\n')
  }, {}, constraintsProps, {
    selectSize: size
  });

  if (autosuggestProps.hidden) {
    return null;
  }

  var suggestions = stateText ? datalist.filter(function (_ref3) {
    var text = _ref3.text,
        value = _ref3.value;
    return text.toLowerCase().match(stateText.toLowerCase());
  }) : datalist;
  return React.createElement("div", {
    className: "sui-FormBuilder-field sui-FormBuilder-AutosuggestSelect sui-FormBuilder-" + autosuggestProps.id
  }, autosuggestProps.label && React.createElement("label", {
    className: "sui-FormBuilder-label",
    htmlFor: autosuggestProps.id
  }, autosuggestProps.label), React.createElement(MoleculeAutosuggestField, autosuggestProps, suggestions.map(function (suggestion, i) {
    return React.createElement(MoleculeAutosuggestOption, {
      key: suggestion.value,
      value: suggestion.text
    }, suggestion.text);
  })));
};

AutosuggestSelect.displayName = 'AutosuggestSelect';
export default AutosuggestSelect;