var _DISPLAYS;

// map the DSL standard to JS: https://docs.mpi-internal.com/scmspain/all--lib-form-builder-docs/form-specification/field/
import { pickFieldById } from '../reducer/fields';
var FIELDS = {
  TEXT: 'text',
  NUMERIC: 'numeric',
  FIELDSET: 'fieldset',
  PICKER: 'picker'
};
var DISPLAYS = (_DISPLAYS = {}, _DISPLAYS[FIELDS.TEXT] = {
  MULTILINE: 'multiline',
  EMAIL: 'email',
  PHONE: 'phone',
  // not part of the DSL standard
  TEXT: 'text',
  // not part of the DSL standard
  DEFAULT: ''
}, _DISPLAYS[FIELDS.NUMERIC] = {
  MONEY: 'money',
  SURFACE: 'surface',
  DEFAULT: ''
}, _DISPLAYS[FIELDS.PICKER] = {
  SWITCH: 'switch',
  AUTOCOMPLETE: 'autocomplete',
  DROPDOWN: 'dropdown',
  RADIO: 'radio',
  CHECKBOX: 'checkbox',
  BUTTON_INLINE: 'button-inline'
}, _DISPLAYS);
var CONSTRAINTS = {
  notnull: 'valueMissing',
  pattern: 'patternMismatch',
  maxlength: 'tooLong',
  minlength: 'tooShort'
};

var checkConstrainstsFactory = function checkConstrainstsFactory(json) {
  return function (_ref) {
    var _document$getElementB;

    var fieldID = _ref["for"],
        all = _ref.all;
    var field = pickFieldById(json.form.fields, fieldID);
    var errorMessages = [];
    var constraints = field.constraints || [];
    var elementValidity = (_document$getElementB = document.getElementById(field.id)) === null || _document$getElementB === void 0 ? void 0 : _document$getElementB.validity; // if element has no validity is not a form element
    // if element has not constraints there is no need to validate

    if (elementValidity && !!constraints) {
      errorMessages = constraints.filter(function (constraint) {
        // for each constraint property defined in the field
        var validityRule = CONSTRAINTS[Object.keys(constraint === null || constraint === void 0 ? void 0 : constraint.property)[0]]; // if true, the constraint is not satisfied, so add the constraint to errors

        if (elementValidity[validityRule]) return true;
        return false;
      }) // return
      .map(function (constraint) {
        return constraint.message;
      });
    } // custom validation: minlength constraint in html textarea is not working as expected, need to handle it manually


    if (field.type === FIELDS.TEXT && field.display === DISPLAYS[FIELDS.TEXT].MULTILINE) {
      var _document$getElementB2, _textAreaHasMinLength;

      var textAreaHasMinLength = field.constraints.find(function (constraint) {
        var _constraint$property;

        return (_constraint$property = constraint.property) === null || _constraint$property === void 0 ? void 0 : _constraint$property.minlength;
      });
      var textAreaValue = (_document$getElementB2 = document.getElementById(field.id)) === null || _document$getElementB2 === void 0 ? void 0 : _document$getElementB2.value;
      if ((textAreaValue === null || textAreaValue === void 0 ? void 0 : textAreaValue.length) < parseInt(textAreaHasMinLength === null || textAreaHasMinLength === void 0 ? void 0 : (_textAreaHasMinLength = textAreaHasMinLength.property) === null || _textAreaHasMinLength === void 0 ? void 0 : _textAreaHasMinLength.minlength)) errorMessages = [].concat(errorMessages, [textAreaHasMinLength.message]);
    } // custom validation: pattern constraint in html input (type=checkbox) is not natively supported, need to handle it manually


    if (field.type === FIELDS.PICKER && field.display === DISPLAYS[FIELDS.PICKER].CHECKBOX) {
      var _document$getElementB3;

      var checkboxValue = JSON.parse((_document$getElementB3 = document.getElementById(field.id)) === null || _document$getElementB3 === void 0 ? void 0 : _document$getElementB3.value);
      var checkboxShouldBeTrueConstraint = field.constraints.find(function (constraint) {
        var _constraint$property2;

        return ((_constraint$property2 = constraint.property) === null || _constraint$property2 === void 0 ? void 0 : _constraint$property2.pattern) === '^true$';
      }); // if should be true according to field, and is false show error message

      if (checkboxShouldBeTrueConstraint && !checkboxValue) errorMessages = [checkboxShouldBeTrueConstraint.message].concat(errorMessages);
    }

    return errorMessages;
  };
};

export { FIELDS, DISPLAYS, CONSTRAINTS, checkConstrainstsFactory };