import _regeneratorRuntime from "@babel/runtime/regenerator";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

/* eslint-disable no-console */
import { operators } from './operators';
import { IN, NIN, EXISTS, NEXISTS, CHANGED, REMOTE } from './constants';
import { changeFieldById, fieldsToQP } from './fields';

var fetch = function fetch(url) {
  return new Promise(function (resolve, reject) {
    var request = new window.XMLHttpRequest();
    request.responseType = 'json';
    request.withCredentials = true;

    request.onreadystatechange = function () {
      if (request.readyState === window.XMLHttpRequest.DONE) {
        if (request.status === 200) {
          resolve(request.response);
        } else {
          reject(Error(request.status));
        }
      }
    };

    request.onerror = function () {
      reject(Error('Network Error'));
    };

    request.open('GET', url, true);
    request.send();
  });
};

export var shouldApplyRule = function shouldApplyRule(fields, changeField) {
  return function (when) {
    // Reduce works as AND operator, all rules should be true
    var isValid = when.reduce(function (acc, rule) {
      // if (rule.id !== changeField) {
      if (!when.some(function (rule) {
        return rule.id === changeField;
      })) {
        return false;
      }

      var isValid;

      switch (rule.operator) {
        case IN:
          isValid = operators.IN(rule.id, rule.value, fields, changeField);
          break;

        case NIN:
          isValid = operators.NIN(rule.id, rule.value, fields, changeField);
          break;

        case EXISTS:
          isValid = operators.EXISTS(rule.id, fields);
          break;

        case NEXISTS:
          isValid = operators.NEXISTS(rule.id, fields);
          break;

        case CHANGED:
          isValid = operators.CHANGED(rule.id, changeField);
          break;

        default:
          console.warn("Unkown operator " + rule.operator);
          isValid = false;
      }

      return acc && isValid;
    }, true);
    return isValid;
  };
};
export var fetchRemoteFields = function fetchRemoteFields(fields, formID, requestInterceptor, urlInterceptor) {
  return (
    /*#__PURE__*/
    function () {
      var _ref = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee(fieldsToChanges) {
        var remoteFieldsToChange;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return Promise.all(Object.entries(fieldsToChanges).filter(function (_ref2) {
                  var field = _ref2[0],
                      nextValue = _ref2[1];
                  return nextValue.remote === REMOTE;
                }).map(function (_ref3) {
                  var field = _ref3[0],
                      nextValue = _ref3[1];
                  var url = urlInterceptor({
                    fieldID: field,
                    fields: fields
                  }) || "https://ptaformbuilder-classifiedads.spain.schibsted.io/fieldrules/" + field + "?" + fieldsToQP(fields, formID);

                  var remote = nextValue.remote,
                      resetValue = _objectWithoutPropertiesLoose(nextValue, ["remote"]);

                  return fetch(url).then(function (json) {
                    var nextJSON = requestInterceptor({
                      url: url,
                      response: json
                    });
                    return [field, _objectSpread({}, resetValue, {}, nextJSON)];
                  })["catch"](function () {
                    console.error("FAIL requesting remote nextValue for " + field);
                    return [field, {}];
                  });
                }));

              case 2:
                remoteFieldsToChange = _context.sent;
                return _context.abrupt("return", remoteFieldsToChange);

              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    }()
  );
};
export var applyRules =
/*#__PURE__*/
function () {
  var _ref4 = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee2(fields, rules, changeField, formID, requestInterceptor, urlInterceptor) {
    var shouldApplyRuleForFieldsAndChangeField, fetchRemoteFieldsForFieldsAndFormID, fieldsToChanges, remoteFieldsToChange, nextFields;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            shouldApplyRuleForFieldsAndChangeField = shouldApplyRule(fields, changeField);
            fetchRemoteFieldsForFieldsAndFormID = fetchRemoteFields(fields, formID, requestInterceptor, urlInterceptor);
            fieldsToChanges = Object.entries(rules).reduce(function (acc, _ref5) {
              var _ref6;

              var idField = _ref5[0],
                  rulesField = _ref5[1];
              // Find works as OR operator, and we want the first match
              var firstMatch = rulesField.find(function (ruleField) {
                return shouldApplyRuleForFieldsAndChangeField(ruleField.when);
              });
              return _objectSpread({}, acc, {}, firstMatch && (_ref6 = {}, _ref6[idField] = _objectSpread({}, firstMatch.then.data, {}, firstMatch.then.remote && {
                remote: REMOTE
              }), _ref6));
            }, {});
            _context2.next = 5;
            return fetchRemoteFieldsForFieldsAndFormID(fieldsToChanges);

          case 5:
            remoteFieldsToChange = _context2.sent;
            nextFields = Object.entries(fieldsToChanges).filter(function (_ref7) {
              var idField = _ref7[0],
                  nextValue = _ref7[1];
              return nextValue !== REMOTE;
            }).reduce(function (acc, _ref8) {
              var idField = _ref8[0],
                  nextValue = _ref8[1];
              return changeFieldById(acc, idField, nextValue);
            }, fields);
            nextFields = remoteFieldsToChange.reduce(function (acc, _ref9) {
              var idField = _ref9[0],
                  nextValue = _ref9[1];
              return changeFieldById(acc, idField, nextValue);
            }, nextFields);
            return _context2.abrupt("return", nextFields);

          case 9:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function applyRules(_x2, _x3, _x4, _x5, _x6, _x7) {
    return _ref4.apply(this, arguments);
  };
}();